@@ -11,7 +11,11 @@
 #include <SoapySDR/Logger.hpp>
 #include <uhd/version.hpp>
 #include <uhd/device.hpp>
+#ifdef UHD_HAS_MSG_HPP
 #include <uhd/utils/msg.hpp>
+#else
+#include <uhd/utils/log_add.hpp>
+#endif
 #include <uhd/usrp/multi_usrp.hpp>
 #include <uhd/property_tree.hpp>
 #include <cctype>
@@ -236,7 +240,7 @@ public:
     int activateStream(SoapySDR::Stream *handle, const int flags, const long long timeNs, const size_t numElems)
     {
         SoapyUHDStream *stream = reinterpret_cast<SoapyUHDStream *>(handle);
-        if (not stream->rx) return SoapySDR::Device::activateStream(handle, flags, timeNs, numElems);
+        if (not stream->rx) return 0; //NOP, does nothing, but not an error
 
         //determine stream mode
         uhd::stream_cmd_t::stream_mode_t mode;
@@ -258,11 +262,15 @@ public:
     int deactivateStream(SoapySDR::Stream *handle, const int flags, const long long timeNs)
     {
         SoapyUHDStream *stream = reinterpret_cast<SoapyUHDStream *>(handle);
-        if (not stream->rx) return SoapySDR::Device::deactivateStream(handle, flags, timeNs);
-        if (flags != 0) return SOAPY_SDR_NOT_SUPPORTED;
+        if (not stream->rx) return 0; //NOP, does nothing, but not an error
 
-        //stop the stream
-        stream->rx->issue_stream_cmd(uhd::stream_cmd_t::STREAM_MODE_STOP_CONTINUOUS);
+        //stop the stream (stop mode might support a timestamp)
+        uhd::stream_cmd_t cmd(uhd::stream_cmd_t::STREAM_MODE_STOP_CONTINUOUS);
+        cmd.stream_now = (flags & SOAPY_SDR_HAS_TIME) == 0;
+        cmd.time_spec = uhd::time_spec_t::from_ticks(timeNs, 1e9);
+
+        //issue command
+        stream->rx->issue_stream_cmd(cmd);
         return 0;
     }
 
@@ -580,7 +588,8 @@ public:
             //read the range from the property tree
             uhd::property_tree::sptr tree = _dev->get_device()->get_tree();
             const std::string path = str(boost::format("/mboards/0/%s_dsps/%u/freq/range") % ((dir == SOAPY_SDR_TX)?"tx":"rx") % channel);
-            return metaRangeToRangeList(tree->access<uhd::meta_range_t>(path).get());
+            if (tree->exists(path)) return metaRangeToRangeList(tree->access<uhd::meta_range_t>(path).get());
+            else return SoapySDR::RangeList(1, SoapySDR::Range(-getSampleRate(dir, channel)/2, getSampleRate(dir, channel)/2));
         }
         return SoapySDR::Device::getFrequencyRange(dir, channel, name);
     }
@@ -825,6 +834,7 @@ private:
 /***********************************************************************
  * Register into logger
  **********************************************************************/
+#ifdef UHD_HAS_MSG_HPP
 static void SoapyUHDLogger(uhd::msg::type_t t, const std::string &s)
 {
     if (s.empty()) return;
@@ -837,6 +847,37 @@ static void SoapyUHDLogger(uhd::msg::type_t t, const std::string &s)
     case uhd::msg::fastpath: SoapySDR::log(SOAPY_SDR_SSI, s); break;
     }
 }
+#else
+static void SoapyUHDLogger(const uhd::log::logging_info &info)
+{
+    //build a log message formatted from the information
+    std::string message;
+
+    if (not info.file.empty())
+    {
+        std::string shortfile = info.file.substr(info.file.find_last_of("/\\") + 1);
+        message += "[" + shortfile + ":" + std::to_string(info.line) + "] ";
+    }
+
+    if (not info.component.empty())
+    {
+        message += "[" + info.component + "] ";
+    }
+
+    message += info.message;
+
+    switch(info.verbosity)
+    {
+    case uhd::log::trace:   SoapySDR::log(SOAPY_SDR_TRACE, message); break;
+    case uhd::log::debug:   SoapySDR::log(SOAPY_SDR_DEBUG, message); break;
+    case uhd::log::info:    SoapySDR::log(SOAPY_SDR_INFO, message); break;
+    case uhd::log::warning: SoapySDR::log(SOAPY_SDR_WARNING, message); break;
+    case uhd::log::error:   SoapySDR::log(SOAPY_SDR_ERROR, message); break;
+    case uhd::log::fatal:   SoapySDR::log(SOAPY_SDR_FATAL, message); break;
+    default: break;
+    }
+}
+#endif
 
 /***********************************************************************
  * Registration
@@ -886,7 +927,11 @@ SoapySDR::Device *make_uhd(const SoapySDR::Kwargs &args)
         "Suggestion: install an ABI compatible version of UHD,\n"
         "or rebuild SoapySDR UHD support against this ABI version.\n"
     ) % UHD_VERSION_ABI_STRING % uhd::get_abi_string()));
+    #ifdef UHD_HAS_MSG_HPP
     uhd::msg::register_handler(&SoapyUHDLogger);
+    #else
+    uhd::log::add_logger("SoapyUHDDevice", &SoapyUHDLogger);
+    #endif
     return new SoapyUHDDevice(uhd::usrp::multi_usrp::make(kwargsToDict(args)), args);
 }
 
