@@ -15,7 +15,11 @@
 #include <uhd/property_tree.hpp>
 #include <uhd/device.hpp>
 #include <uhd/convert.hpp>
+#ifdef UHD_HAS_MSG_HPP
 #include <uhd/utils/msg.hpp>
+#else
+#include <uhd/utils/log.hpp>
+#endif
 #include <uhd/types/sensors.hpp>
 #include <uhd/types/ranges.hpp>
 #include <uhd/usrp/mboard_eeprom.hpp>
@@ -690,18 +694,18 @@ class UHDSoapyTxStream : public uhd::tx_streamer
 {
 public:
     UHDSoapyTxStream(SoapySDR::Device *d, const uhd::stream_args_t &args):
+        _active(false),
         _device(d),
         _stream(make_stream(d, SOAPY_SDR_TX, args)),
         _nchan(std::max<size_t>(1, args.channels.size())),
         _elemSize(uhd::convert::get_bytes_per_item(args.cpu_format))
     {
         _offsetBuffs.resize(_nchan);
-        _device->activateStream(_stream);
     }
 
     ~UHDSoapyTxStream(void)
     {
-        _device->deactivateStream(_stream);
+        if (_active) _device->deactivateStream(_stream);
         _device->closeStream(_stream);
     }
 
@@ -722,6 +726,13 @@ public:
         const double timeout = 0.1
     )
     {
+        //perform activation at the latest/on the first call to send
+        if (not _active)
+        {
+            _device->activateStream(_stream);
+            _active = true;
+        }
+
         size_t total = 0;
         const long long timeNs(md.time_spec.to_ticks(1e9));
 
@@ -738,6 +749,13 @@ public:
             total += ret;
         }
 
+        //implement deactivation hook for very last sample consumed on end of burst
+        if (_active and md.end_of_burst and total == nsamps_per_buff)
+        {
+            _device->deactivateStream(_stream);
+            _active = false;
+        }
+
         return total;
     }
 
@@ -793,6 +811,7 @@ public:
     }
 
 private:
+    bool _active;
     SoapySDR::Device *_device;
     SoapySDR::Stream *_stream;
     const size_t _nchan;
@@ -820,8 +839,10 @@ bool UHDSoapyDevice::recv_async_msg(uhd::async_metadata_t &md, double timeout)
  **********************************************************************/
 static void UHDSoapyLogger(const SoapySDR::LogLevel logLevel, const char *message)
 {
+    #define component "UHDSoapyDevice"
     switch(logLevel)
     {
+    #ifdef UHD_HAS_MSG_HPP
     case SOAPY_SDR_FATAL:
     case SOAPY_SDR_CRITICAL:
     case SOAPY_SDR_ERROR: UHD_MSG(error) << message << std::endl; break;
@@ -831,6 +852,17 @@ static void UHDSoapyLogger(const SoapySDR::LogLevel logLevel, const char *messag
     case SOAPY_SDR_DEBUG:
     case SOAPY_SDR_TRACE: UHD_MSG(status) << message << std::endl; break;
     case SOAPY_SDR_SSI: UHD_MSG(fastpath) << message << std::flush; break;
+    #else
+    case SOAPY_SDR_FATAL:
+    case SOAPY_SDR_CRITICAL:  UHD_LOG_FATAL(component, message); break;
+    case SOAPY_SDR_ERROR:     UHD_LOG_FATAL(component, message); break;
+    case SOAPY_SDR_WARNING:   UHD_LOG_WARNING(component, message); break;
+    case SOAPY_SDR_NOTICE:
+    case SOAPY_SDR_INFO:      UHD_LOG_INFO(component, message); break;
+    case SOAPY_SDR_DEBUG:
+    case SOAPY_SDR_TRACE:     UHD_LOG_TRACE(component, message); break;
+    case SOAPY_SDR_SSI:       UHD_LOG_FASTPATH(message); break;
+    #endif
     }
 }
 
